from django.http import HttpResponse
from django.views.generic import View


class Controller(View):
    url_params = None

    """
    Base controller.
    """
    def send(self, data):
        return HttpResponse(data)



import os
from django.conf import settings
import sys


def import_module(module_name):
    """
    Import module and return it.
    """
    __import__(module_name)
    return sys.modules[module_name]


def listdir(dir_name, get_dirs=None, get_files=None, hide_ignored=False):
    """
    Return list of all dirs and files inside given dir.

    Also can filter contents to return only dirs or files.

    Args:
    - dir_name: Which directory we need to scan (relative)
    - get_dirs: Return dirs list
    - get_files: Return files list
    - hide_ignored: Exclude files and dirs with initial underscore
    """
    if get_dirs is None and get_files is None:
        get_dirs = True
        get_files = True

    source_dir = os.path.join(settings.BASE_DIR, dir_name)

    dirs = []

    for dir_or_file_name in os.listdir(source_dir):
        path = os.path.join(source_dir, dir_or_file_name)
        if hide_ignored and dir_or_file_name.startswith('_'):
            continue
        is_dir = os.path.isdir(path)
        if get_dirs and is_dir or get_files and not is_dir:
            dirs.append(dir_or_file_name)

    return dirs



"""
Auto-discoverable urls.
"""
import re
from django.conf.urls import url
from django.contrib import admin
from .controllers import Controller
from .tools import import_module
from .tools import listdir


def auto_discover():
    """
    Auto-map urls from controllers directory.

    Ignored only files and classes that start from underscore.
    """
    urls = [
        url(r'^admin/', admin.site.urls),
    ]

    # TODO: we can create python package to have a lot of controllers
    # in separate files
    def get_controllers(module_name):
        """Return list of controllers in a module."""
        module = import_module('controllers.{}'.format(module_name))
        controllers = []
        for obj_name in dir(module):
            # we ignore import of Controller and hidden names
            if obj_name.startswith('_') or obj_name == 'Controller':
                continue
            obj = getattr(module, obj_name)
            # include only controllers
            if issubclass(obj, Controller):
                controllers.append(obj)
        return controllers

    def controller_to_path(controller):
        """
        Convert controller's name to a valid path.

        Make url in lower case by replace capital letters to small
        and adding underscore between words.
        """
        words = re.findall('[A-Z][a-z]*', controller.__name__)
        if words[-1] == 'Controller':
            del words[-1]
        # transform words to a url address
        url_path = '_'.join(words).lower()
        # main controller is a root handler
        if url_path == 'main':
            url_path = ''
        return url_path

    # load all controllers (excluding main controllers)
    for file_name in listdir('controllers', get_files=True, hide_ignored=True):
        # remove .py extension from file name
        app_name = file_name.split('.', 1)[0]
        # we will include main controller at the end
        if app_name == 'main':
            continue
        # add ulr for each controller
        for controller in get_controllers(app_name):
            url_path = controller_to_path(controller)
            urls.append(url(
                r'^{}/{}$'.format(app_name, url_path),
                controller.as_view()
            ))

    # add urls for main controllers
    for controller in get_controllers('main'):
        url_path = controller_to_path(controller)
        # map urls to a root path
        urls.append(url(
            r'^{}$'.format(url_path),
            controller.as_view()
        ))

    return urls



"""
Django(mini)
====================

**Django(mini)** - Infrastructure for rapid development on Django.


Quick start
-------------

Read `Quick Start <https://github.com/djangomini/djangomini>`_ on GitHub.


**Don't forget to star project on
`github.com <https://github.com/djangomini/djangomini>`
to tell us that you like it ;-)**
"""

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


version = '1.0.0a1'


setup(
    name='djangomini',
    version=version,
    url='https://github.com/djangomini/djangomini',
    license='MIT',
    author='Anton Danilchenko',
    author_email='anton@danilchenko.me',
    description='Django(mini) - Infrastructure for rapid development on Django.',
    keywords='django web framework',
    long_description=__doc__,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    packages=[
        'djangomini',
        # in future:
        # 'djangomini.controllers',
        # 'djangomini.models',
        # 'djangomini.views'
    ],
    install_requires=['django'],
    include_package_data=True,
    zip_safe=False,
    platforms='any'
)
